#!/bin/bash
# Copyright (C) 2004-2013 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of datasid
#
# collect-agent is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# Exit status:

# 0 - Success
# 1 - Error to rotate log file
# 2 - Error to edit crontab
# 3 - Error when updating
# 4 - Error to collect system data
# 5 - Error to download telecentroInfo
# 6 - Error when loading configuration
# 7 - Error to send collected data

#------------------------------------------------------------------------------
# Function: rotateLogFile
# If the log file is out-of-date (set by $2), execute savelog(1) to rotate it.
# Parameters:
# $1: log file name
# $2: time in days to expire
# $3: save $3 versions of the log file
function rotateLogFile()
{
    find "${LOGDIR}" -type f -name "$1" -atime +$2 -exec savelog -c $3 {} \; ||
    return 1

    return 0
}

#------------------------------------------------------------------------------
# Function: checkUpdate
# Verify and download the new version of the agent.
function checkUpdate()
{
    if test "${FIRST_EXECUTION}"; then
        updateLink="$(${PREFIX}/bin/datasid-client.sh --update 0.0.0)"
    else
        updateLink="$(${PREFIX}/bin/datasid-client.sh --update ${VERSION})"
    fi

    returnValue=$?

    if test ${returnValue} -eq 4; then
        test -f "${DATADIR}/datasid-update.run" || rm -f ${DATADIR}/datasid-update.run

        ${PREFIX}/bin/datasid-wget.sh ${updateLink} \
                -O ${DATADIR}/datasid-update.run || return 1

        cd ${DATADIR}

        # If $FIST_EXECUTION is set, do not run the update. Otherwise, run the
        # update and return 1 in case of error
        test "${FIRST_EXECUTION}" || sudo bash ${DATADIR}/datasid-update.run || return 1

        rm -f ${DATADIR}/datasid-update.run

        # Return 0 if this is the first execution of the agent
        test "${FIRST_EXECUTION}" && return 0
    fi

    return ${returnValue}
}

#------------------------------------------------------------------------------
# Function: schedule
# Check scheduling in the user's crontab and write the new schedule in case
# there wasn't one or if it was different.
function schedule()
{
    if test -f "${CONFDIR}/telecentroInfo"; then
        MIN=$(( 10#$(grep superid ${CONFDIR}/telecentroInfo | awk -F'=' '{print $2}' | sed "s/[^0-9]//g") % 60 ))
    else
        MIN=$(( $RANDOM % 60 ))
    fi

    cronAgent="${MIN} * * * * ${PREFIX}/bin/datasid-check.sh && ${PREFIX}/bin/datasid-agent.sh && ${PREFIX}/bin/datasid-network-usage.sh"
    cronNetmon="0-59/5 * * * * ${PREFIX}/bin/datasid-netmon.sh"
    cronReboot="@reboot ${PREFIX}/bin/datasid-agent.sh"

    fullCron="$(printf "${cronAgent}\n${cronNetmon}\n${cronReboot}\n")"

    if test "$(crontab -l)" != "${fullCron}"; then
        printf "${fullCron}\n" | crontab - || return 1
    fi

    return 0
}

#------------------------------------------------------------------------------
# Function: elapsedTime
# Verify if ${DELAY} seconds have elapsed since agent last execution.
# Return true or false.
function elapsedTime()
{
    test -f "${DATADIR}/lastexec" || return 0

    atSeconds=$(date +%s)
    oldSeconds=$(cat ${DATADIR}/lastexec)

    (( (${atSeconds}-${oldSeconds}) >= ${DELAY} ))

    return $?
}

# set PREFIX
export PREFIX="$(readlink -f "$(dirname $0)/../")"

# run datasid-common.sh
source "$(readlink -f "$(dirname $0)")/datasid-common.sh" || exit 6

if test -z "${PREFIX}"; then
    date +"%F %T - ERROR: Prefix not set."
    exit 1
fi

exec >> ${LOGFILE} 2>&1

# Rotate the log file if it has already expired
if ! rotateLogFile '*.log' ${DAYSTOEXPIRELOG} 12; then
    date +"%F %T - Error while rolling log file."
    exit 1
fi

date +"%F %T - Agent started."

# Edit crontab if it isn't correct
if ! schedule; then
    date +"%F %T - Error to edit crontab."
    exit 2
fi

date +"%F %T - Start of update process execution."

checkUpdate
updateStatus=$?

# Updating the agent, just exit (normal behaviour)
test ${updateStatus} -eq 4 && exit 0

if test ${updateStatus} -ne 0; then
    date +"%F %T - Error while updating agent."
    exit 3
fi

date +"%F %T - Update process terminated successfully."

# Download telecentroInfo from webservice
if ! ${PREFIX}/bin/datasid-wget.sh 192.168.0.253/api/datasid -O ${DATADIR}/telecentroInfo -t $UPDATETRIES -T 10; then
    date +"%F %T - Error while downloading telecentroInfo."
#	exit 5
else
	# Check if a change has occurred in the webservice
	if ! test -f ${CONFDIR}/telecentroInfo -a '$(md5sum ${DATADIR}/telecentroInfo)' = '$(md5sum ${CONFDIR}/telecentroInfo)'; then
		date +"%F %T - Getting new telecentroInfo"
		mv -f ${DATADIR}/telecentroInfo ${CONFDIR}/telecentroInfo
	fi
fi

rm -f ${DATADIR}/telecentroInfo

# Check if telecentroInfo is configured
if ! test -f ${CONFDIR}/telecentroInfo; then
	date +"%F %T - telecentroInfo not found in ${CONFDIR}/telecentroInfo, waiting configuration."
	exit 8
fi

# Collect and send system data if elapsed enough time or if it's the first
# execution
if elapsedTime; then

    # Collect system data
    if ! ${PREFIX}/bin/datasid-xml-generator "${PREFIX}/gen-scripts" "${DATADIR}/collect.xml"; then
        date +"%F %T - ERROR: Failed to collect system data."
        exit 4
    fi

    # Send collected data
    if ! ${PREFIX}/bin/datasid-client.sh --send-inventory "${DATADIR}/collect.xml"; then
        date +"%F %T - ERROR: Failed to send collected data."
        exit 7
    fi

    # Record the current execution time to be used by elapsedTime()
    date +%s > ${DATADIR}/lastexec
fi

date +"%F %T - Agent terminated sucessfully."
exit 0
