/* Copyright (C) 2004-2013 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA. */

/*---------------------------------------------------------------------------*/

#include "xmlgenerator.h"

int main(int argc, char **argv)
{
    char *rootDir;
    char *xmlFile;

    if (argc < 3)
    {
        ERROR("USAGE: %s <directory> <XML file>\n", argv[0]);
        ERROR("  directory: Directory with generator scripts.\n");
        ERROR("  XML file: Name of XML file to create/update.\n");
        return 1; /* Parameter error */
    }
    else
    {
        if (! ((rootDir = argv[1]) && strcmp(rootDir,"")))
        {
            ERROR("ERROR: The directory name is invalid.\n");
            return 2;
        }
        xmlFile = argv[2];
    }

    /*-------------------------------------------------------*/

    generateXML(rootDir, xmlFile);

    return printErrorMsgs(DEBUG_ALL);
}

/*---------------------------------------------------------------------------*/
/*
 * Function: setErrorFlag
 * Description: Assigns the flag value in statusRunning to debug execution.
 */
int setErrorFlag(const char flag)
{
    return (statusRunning |= flag);
}

/*---------------------------------------------------------------------------*/
/*
 * Function: unsetErrorFlag
 * Description: Unassigns the flag value in statusRunning.
 */
int unsetErrorFlag(const char flag)
{
    if (statusRunning & flag)
    {
        return (statusRunning ^= flag);
    }

    return statusRunning;
}

/*---------------------------------------------------------------------------*/
/*
 * Function: printErrorMsgs
 * Description: Prints error messages according to flags assigned in
 * statusRunning and debug mode. The debug mode is defined by flagsEnabled
 * parameter.
 */
int printErrorMsgs(const char flagsEnabled)
{
    if (flagsEnabled & statusRunning & MEM_ALLOC)
    {
        ERROR("ERROR: Could not allocate memory.\n");
    }

    if (flagsEnabled & statusRunning & CMD_RESULT)
    {
        ERROR("ERROR: The collect script returned an error code.\n");
    }

    if (flagsEnabled & statusRunning & PERM_DENIED_CMD)
    {
        ERROR("ERROR: Permission denied to execute collect script.\n");
    }

    if (flagsEnabled & statusRunning & ARGS)
    {
        ERROR("ERROR: Invalid arguments.\n");
    }

    if (flagsEnabled & statusRunning & CREATE_XML_DOC)
    {
        ERROR("ERROR: Could not create XML document in memory.\n");
    }

    if (flagsEnabled & statusRunning & SAVE_XML_FILE)
    {
        ERROR("ERROR: Could not save XML file.\n");
    }

    if (flagsEnabled & statusRunning & PARSE_COLLECT)
    {
        ERROR("ERROR: Collect data error.\n");
    }

    return statusRunning;
}

/*---------------------------------------------------------------------------*/
/*
 * Function: allocString
 * Description: Allocate or reallocate memory for a string of size chars,
 * and save pointer to **string.
 */
static int allocString(char **string, const int size)
{
    if (*string)
    {
        return ((*string = (char*)realloc(*string, sizeof(char) * size))
            != NULL);
    }
    else
    {
        return ((*string = (char*)malloc(sizeof(char) * size)) != NULL);
    }
}

/*---------------------------------------------------------------------------*/
/*
 * Function: concatString
 * Description: Concatenates strings *string1 with *string2, and saves the
 * result in **result. Allocates the memory needed for string result.
 */
static int concatString(char **result, const char *string1, const char
        *string2)
{
    int destTam = (strlen(string1) + strlen(string2) + 1) * sizeof(char),
        execStrCpy = 1, diffResultStr2 = 1;
    char *tempStr = NULL;

    if (*result)
    {
        execStrCpy = strcmp(*result, string1);

        if (! strcmp(*result, string2))
        {
            if (! allocString(&tempStr, strlen(string2)))
            {
                return 0;
            }

            strcpy(tempStr, string2);
            diffResultStr2 = 0;
        }

        if (! allocString(result, destTam))
        {
            return 0;
        }
    }
    else if (! allocString(result, destTam))
    {
        return 0;
    }

    /* If the string1 is equal to result, don't execute strcpy */
    if (execStrCpy)
    {
        strcpy(*result, string1);
    }
    /* If the string2 is equal the result, has exec strcat with the string
     * tempStr. */
    if (diffResultStr2)
    {
        strcat(*result, string2);
    }
    else
    {
        strcat(*result, tempStr);
        free(tempStr);
    }

    return 1;
}

/*---------------------------------------------------------------------------*/
/*
 * Function: nodeName
 * Description: Using a '.' as delimiter, return the first part of string.
 */
static char *nodeName(char *fullName)
{
    char *temp;
    int tamSubString = 1;

    for (temp = fullName;
            *temp != '.' && *temp != '\0'; temp++, tamSubString++);

    temp = NULL;
    if (! allocString(&temp, tamSubString))
    {
        return NULL;
    }
    strncpy(temp, fullName, tamSubString-1);

    temp[tamSubString-1] = '\0';

    return temp;

}

/*---------------------------------------------------------------------------*/
/*
 * Function: attribData
 * Description: Using a '.' as delimiter, return the last part of string.
 */
static char *attribData(const char *fileName)
{
    char *temp = strstr(fileName, ".");

    return (temp ? ++temp : NULL);
}
/*---------------------------------------------------------------------------*/
/*
 * Function: execCommand
 * Description: Execute the *command and write the stdout of this command in
 * **resultExec.
 */
static int execCommand(const char *command,
        char **resultExec)
{
    int index, execTam = STR_INI_SIZE;
    FILE *result;

    printf("Running %s\n", command);

    /* Execute command */
    if (! (result = popen(command,"r")))
    {
        return 0;
    }

    /* alloc and realloc **resultExec (if necessary) and write the stdout of
     * the *command */
    if (! allocString(resultExec, execTam))
    {
        return 0;
    }
    for (index = 0; (resultExec[0][index] = fgetc(result)) != '\n'
            && resultExec[0][index] != EOF; index++)
    {
        if (index == execTam-1)
        {
            execTam *= 2;
            if (! allocString(resultExec, execTam))
            {
                return 0;
            }
        }
    }

    resultExec[0][index] = '\0';
    if (pclose(result))
    {
        return 0;
    }

    return 1;
}
/*---------------------------------------------------------------------------*/
static int parserXml(const char *currentDirName, xmlTextWriterPtr *xmlWriter)
{
    struct dirent **contentDir;
    struct stat contentDirStat;
    char *tmpDirName, *tempNodeName, *resultExec;

    int i, nElements = scandir(currentDirName, &contentDir, 0, alphasort);

    if (nElements == -1)
    {
        return 0;
    }

    for (i = 0; i < nElements ; i++)
    {
        /* Ignore directories "." and ".." */
        if (strcmp(contentDir[i]->d_name, ".")
                && strcmp(contentDir[i]->d_name, "..") )
        {
            tempNodeName = tmpDirName = NULL;
            tempNodeName = nodeName(contentDir[i]->d_name);

            /* Concatenate current directory path with contentDir->d_name */
            if (! concatString(&tmpDirName, currentDirName, "/"))
            {
                return 0;
            }
            if (! concatString(&tmpDirName, tmpDirName, contentDir[i]->d_name))
            {
                return 0;
            }

            /* Parse the type of current file */
            if (lstat(tmpDirName, &contentDirStat))
            {
                return 0;
            }

            /* Macro S_ISDIR returns true if st_mode is directory */
            if (S_ISDIR(contentDirStat.st_mode))
            {
                /* Start element with name *tempNodeName */
                if (0 > xmlTextWriterStartElement(*xmlWriter,
                        BAD_CAST tempNodeName))
                {
                    return 0;
                }

                /* If there is an attribute in current directory name,
                 * write an attribute "value" with the value
                 * attribData(contentDir->d_name) in current element */
                if (attribData(contentDir[i]->d_name))
                {
                    if (0 > xmlTextWriterWriteAttribute(*xmlWriter,
                                BAD_CAST "value",
                                BAD_CAST attribData(contentDir[i]->d_name)))
                    {
                        return 0;
                    }
                }

                /* Entry in directory contentDir->d_name */
                if (! parserXml(tmpDirName, xmlWriter))
                {
                    return 0;
                }

                /* Close element *tempNodeName */
                if (0 > xmlTextWriterEndElement(*xmlWriter))
                {
                    return 0;
                }

            }
            else
            {
                /* If current file is regular then it will be executed.
                 * resultExec get stdout result of execution */

                resultExec=NULL;

                if (! execCommand(tmpDirName, &resultExec))
                {
                    setErrorFlag(CMD_RESULT);
                    return 0;
                }

                if (0 > xmlTextWriterStartElement(*xmlWriter,
                       BAD_CAST tempNodeName))
                {
                    return 0;
                }
                /*if (0 > xmlTextWriterWriteAttribute(*xmlWriter,
                            BAD_CAST "type",
                            BAD_CAST attribData(contentDir[i]->d_name)))
                {
                    return 0;
                }*/


                if (strcmp(attribData(contentDir[i]->d_name), "tree"))
                {
                    if (0 > xmlTextWriterWriteAttribute(*xmlWriter,
                                BAD_CAST "value", BAD_CAST resultExec))
                    {
                        return 0;
                    }
                }
                else
                {
                    if (0 > xmlTextWriterWriteRaw(*xmlWriter,
                                BAD_CAST resultExec))
                    {
                        return 0;
                    }
                }


                if (0 > xmlTextWriterEndElement(*xmlWriter))
                {
                    return 0;
                }

                free(resultExec);
            }

            free(tmpDirName);
            free(tempNodeName);
        }
    }

    return 1;
}

/*---------------------------------------------------------------------------*/
/*
 * Function: generateXML
 * Description: Parse Struct directories and executables children of
 * the *rootDirName, generating a DOM tree with the results.
 */
int generateXML(const char *rootDirName, const char *xmlFile)
{

    xmlDocPtr xmlDocument;
    xmlTextWriterPtr xmlWriter;

    if (! (xmlDocument = xmlNewDoc(BAD_CAST XML_DEFAULT_VERSION)))
    {
        return setErrorFlag(CREATE_XML_DOC);
    }

    if (! (xmlWriter = xmlNewTextWriterTree(xmlDocument, NULL, 0)))
    {
        return setErrorFlag(CREATE_XML_DOC);
    }

    if (0 > xmlTextWriterStartDocument(xmlWriter, NULL, MY_ENCODING, NULL))
    {
        return setErrorFlag(CREATE_XML_DOC);
    }

    if (! parserXml(rootDirName, &xmlWriter))
    {
        return setErrorFlag(PARSE_COLLECT);
    }

    if (! xmlTextWriterEndDocument(xmlWriter))
    {
        return setErrorFlag(CREATE_XML_DOC);
    }

    xmlFreeTextWriter(xmlWriter);

    /* Generate XML File */
    if (0 > xmlSaveFileEnc(xmlFile, xmlDocument, MY_ENCODING))
    {
        return setErrorFlag(SAVE_XML_FILE);
    }

    xmlFreeDoc(xmlDocument);

    return 1;
}
/*---------------------------------------------------------------------------*/
