-- loads the table dim_inventory with inventories of new computers
-- or inventories that had been modified.

CREATE OR REPLACE FUNCTION to_machine_type_enum(i INTEGER) RETURNS machine_type_enum AS $$
SELECT CASE WHEN $1 = 0 THEN 'client'::machine_type_enum
	    WHEN $1 = 1 THEN 'server'::machine_type_enum
	    END as machine_type;
$$ LANGUAGE SQL;


CREATE OR REPLACE FUNCTION load_dim_inventory() returns void as $$
DECLARE
	inv_row record;
	date_row record;
	start_ts TIMESTAMP;
	total_rows INTEGER;
	is_new BOOLEAN;
BEGIN

	-- getting initial timestamp
	start_ts = CLOCK_TIMESTAMP();
	total_rows := 0;

	-- since we removed multiple inventories on the same date on sa_sanitize,
	-- looping one day at a time will ensure that we won't have more than one
	-- inventory from the same computer. 
	FOR date_row IN SELECT DISTINCT contact_date FROM sa_inventory WHERE valid = '1' ORDER BY contact_date
	LOOP

		-- I think the better way to compare the current inventory and the new
		-- one is to iterate over each record. Using one batch insert will
		-- certainly perform way faster, but here we have more flexibility.
		FOR inv_row IN 
			SELECT
				s.tl_id as new_tl_id,
				s.macaddr as new_macaddr,
				s.contact_date as new_contact_date,
				s.machine_type as new_machine_type,
				s.disk1_model as new_disk1_model,
				s.disk1_size as new_disk1_size,
				s.disk1_used as new_disk1_used,
				s.disk2_model as new_disk2_model,
				s.disk2_size as new_disk2_size,
				s.disk2_used as new_disk2_used,
        	        	s.memory as new_memory,
				s.processor as new_processor,
				s.os_type as new_os_type,
				s.os_distro as new_os_distro,
				s.os_kernel as new_os_kernel,

				d.id as cur_id,
				d.tl_id as cur_tl_id,
				d.load_date as cur_load_date,
				d.disk1_model as cur_disk1_model,
				d.disk1_size as cur_disk1_size,
				d.disk1_used as cur_disk1_used,
				d.disk2_model as cur_disk2_model,
				d.disk2_size as cur_disk2_size,
				d.disk2_used as cur_disk2_used,
        		        d.memory as cur_memory,
				d.processor as cur_processor,
				d.os_type as cur_os_type,
				d.os_distro as cur_os_distro,
				d.os_kernel as cur_os_kernel
			FROM sa_inventory s LEFT JOIN 
				(SELECT * FROM dim_inventory WHERE is_current = '1') as d
			ON s.tl_id = d.tl_id AND s.macaddr::macaddr = d.macaddr
			WHERE s.contact_date = date_row.contact_date AND s.valid = '1' 
		LOOP


			-- in which cases we shouldn't update the table
			-- Note that since disk2_* can be null, the logic is a little
			-- more complicated... The other columns are NOT NULL.
			IF     (inv_row.cur_id IS NOT NULL AND
				inv_row.new_tl_id IS NOT NULL AND
	                	inv_row.new_memory     >= inv_row.cur_memory * 0.9 AND
				inv_row.new_memory     <= inv_row.cur_memory * 1.1 AND
				inv_row.new_processor   = inv_row.cur_processor    AND
				inv_row.new_os_type     = inv_row.cur_os_type      AND
				inv_row.new_os_distro   = inv_row.cur_os_distro    AND
				inv_row.new_os_kernel   = inv_row.cur_os_kernel    AND
				inv_row.new_disk1_model = inv_row.cur_disk1_model  AND
				inv_row.new_disk1_size >= inv_row.cur_disk1_size * 0.9 AND
				inv_row.new_disk1_size <= inv_row.cur_disk1_size * 1.1 AND
				inv_row.new_disk1_used >= inv_row.cur_disk1_used * 0.9 AND
				inv_row.new_disk1_used <= inv_row.cur_disk1_used * 1.1 AND
				(
					(inv_row.new_disk2_model IS NULL AND inv_row.cur_disk2_model IS NULL)
					OR
					inv_row.new_disk2_model = inv_row.cur_disk2_model
				) AND
				(
					(inv_row.new_disk2_size IS NULL AND inv_row.cur_disk2_size IS NULL)
					OR
					(inv_row.new_disk2_size >= inv_row.cur_disk2_size * 0.9 AND
					inv_row.new_disk2_size <= inv_row.cur_disk2_size * 1.1)
				) AND
				(
					(inv_row.new_disk2_used IS NULL AND inv_row.cur_disk2_used IS NULL)
					OR
					(inv_row.new_disk2_used >= inv_row.cur_disk2_used * 0.9 AND
					inv_row.new_disk2_used <= inv_row.cur_disk2_used * 1.1)
				))
			THEN
				CONTINUE;
			END IF;

			-- if this inventory is older than the current, ignore it. Probably
			-- the nicest way here would be to insert the historical data, but
			-- that would complicate the loading function too much and slow all 
			-- things down. 
			IF (inv_row.new_contact_date <= inv_row.cur_load_date) THEN
				CONTINUE;
			END IF;

			-- if cur_id is NULL then it's a new record. Marking it accordingly..
			IF (inv_row.cur_id IS NULL) THEN
				is_new := '1';
			ELSE
				is_new := '0';
			END IF;

			-- If we get here, we decided to update the record. First insert
			-- the new inventory
			INSERT INTO dim_inventory (tl_id, macaddr, load_date, machine_type,
				disk1_model, disk1_size, disk1_used, disk2_model,
				disk2_size, disk2_used, memory, processor, os_type,
				os_distro, os_kernel, is_first, is_current) VALUES
				(inv_row.new_tl_id, inv_row.new_macaddr::macaddr,
				inv_row.new_contact_date, to_machine_type_enum(inv_row.new_machine_type),
				inv_row.new_disk1_model, inv_row.new_disk1_size,
				inv_row.new_disk1_used, inv_row.new_disk2_model,
				inv_row.new_disk2_size, inv_row.new_disk2_used,
				inv_row.new_memory, inv_row.new_processor,
				inv_row.new_os_type, inv_row.new_os_distro,
				inv_row.new_os_kernel, (SELECT CASE WHEN is_new THEN B'1' ELSE B'0' END), '1');

			-- only for logging purposes
			total_rows := total_rows + 1;

			-- Since we only insert newer inventories, we must update the 
			-- 'is_current' flag of the last inventory. Note that maybe we 
			-- don't have a previous inventory to update
			IF (inv_row.cur_id IS NOT NULL) THEN
				UPDATE dim_inventory SET is_current = '0' WHERE
					id = inv_row.cur_id;
			END IF;

		END LOOP;
	END LOOP;

	-- logging
	INSERT INTO control (name, start_time, end_time, total) 
		VALUES ('loading dim_inventory', start_ts, CLOCK_TIMESTAMP(), total_rows);

END;
$$ language plpgsql;
