/**
 * The functions on this file return the number of computers of a given
 * 'project' that contacted the server in the last 10 days (green), 10
 * to 30 days (yellow), or more than 30 days (red). You can also provide
 * 'base_date' to refer to the situation in a particular date.
 */


/**
 * Returns the total amount of computers of the given 'project' and their
 * last contact date status.
 *
 * @param in project_enum proj The type of computer/machine
 * @param in DATE base_date The particular date until which we're
 * 	interested. Default value is today.
 * @return TABLE(bigint, bigint, bigint)
 */
DROP FUNCTION inventory_brazil(machine_type_enum, DATE);
CREATE OR REPLACE FUNCTION inventory_brazil(machtype machine_type_enum, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("Total de maquinas" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		COUNT(*)
	FROM
		(SELECT
			DISTINCT macaddr, tl_id
		FROM
			aggr_availability
		WHERE
			machine_type = machtype
		AND
			base_date <= b_date
	) AS a;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' in the
 * current date and the last three months.
 *
 * TODO: generalize the number of months and create the 'dates' temp table
 * in a more elegant manner.
 *
 * @param in project_enum proj The type of computer/machine
 * @return TABLE(date, bigint, bigint, bigint)
 */
DROP FUNCTION inventory_brazil_monthly_history(machine_type_enum);
CREATE OR REPLACE FUNCTION inventory_brazil_monthly_history(machtype machine_type_enum) RETURNS
	TABLE ("Data" text, "Total de maquinas" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		get_month_name(extract(month from base_date)::integer)::text,
		count(*)
	FROM
		aggr_availability
	WHERE
		machine_type = machtype
	GROUP BY
		base_date
	ORDER BY base_date;
END;
$$ language plpgsql;


/**
 * Returns the total amount of computers of the given 'project' and their
 * last contact date status grouped by region.
 *
 * @param in project_enum proj The type of computer/machine
 * @param in DATE base_date The particular date until which we're
 * 	interested. Default value is today.
 * @return TABLE(varchar, bigint, bigint, bigint)
 */
DROP FUNCTION inventory_region(machine_type_enum, DATE);
CREATE OR REPLACE FUNCTION inventory_region(machtype machine_type_enum, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("Região" varchar, "Total de maquinas" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		region,
		COUNT(*)
	FROM
		(SELECT macaddr, tl_id, region
		FROM aggr_availability
		WHERE base_date <= b_date AND
		machine_type = machtype
		GROUP BY macaddr, tl_id, region) AS a
	GROUP BY region;
END;
$$ language plpgsql;

DROP FUNCTION inventory_region_monthly_history(machine_type_enum, text);
CREATE OR REPLACE FUNCTION inventory_region_monthly_history(machtype machine_type_enum, region_in text) RETURNS
	TABLE ("Data" text, "Total de maquinas" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		get_month_name(extract(month from base_date)::integer)::text,
		COUNT(*)
	FROM
		(SELECT
			macaddr, tl_id, base_date
		FROM
			aggr_availability
		WHERE
			machine_type = machtype AND region = region_in
		GROUP BY
			macaddr, tl_id, base_date) AS a
	GROUP BY base_date
	ORDER BY base_date;
END;
$$ language plpgsql;

DROP FUNCTION inventory_state(machine_type_enum, text, date);
CREATE OR REPLACE FUNCTION inventory_state(machtype machine_type_enum, region_in text, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("Estado" varchar, "Total de maquinas" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		state,
		COUNT(*)
	FROM
		(SELECT
			state, macaddr, sch_inep
		FROM
			aggr_availability
		WHERE
			machine_type = machtype AND region = region_in AND base_date <= b_date
		GROUP BY
			sch_inep, macaddr, state) AS a
	GROUP BY state
	ORDER BY state;
END;
$$ language plpgsql;

drop function inventory_state_monthly_history(proj project_enum, state_in text);
CREATE OR REPLACE FUNCTION inventory_state_monthly_history(proj project_enum, state_in text) RETURNS
	TABLE ("Data" text, "Total de maquinas" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		get_month_name(extract(month from base_date)::integer)::text,
		COUNT(*)
	FROM
		(SELECT
			macaddr,
			sch_inep,
			base_date
		FROM
			aggr_availability
		WHERE
			state = state_in AND
			project = proj
		GROUP BY
			macaddr, sch_inep, base_date) AS a
	GROUP BY base_date ORDER BY base_date;
END;
$$ language plpgsql;

DROP FUNCTION inventory_city(project_enum, text, DATE);
CREATE OR REPLACE FUNCTION inventory_city(proj project_enum, state_in text, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("Cidade" varchar, "Total de maquinas" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		city,
		count(*)
	FROM
		(SELECT city
		FROM
			aggr_availability
		WHERE
			project = proj AND
			state = state_in AND
			base_date <= b_date
		GROUP BY city, macaddr, sch_inep) as a
	GROUP BY city
	ORDER BY city;
END;
$$ language plpgsql;

DROP FUNCTION inventory_report(proj project_enum, region_in text, state_in text, city_in text);
CREATE OR REPLACE FUNCTION inventory_report(proj project_enum, region_in text, state_in text, city_in text) RETURNS
	TABLE (data_atual text, regiao text, inv_estado character varying, cidade text,
	escola text, maquina macaddr, data date, SO text, processor text,
	memoria text, HD text) AS $$
BEGIN
	RETURN QUERY SELECT to_char((SELECT max(base_date) FROM aggr_availability), 'DD/MM/YYYY'),
		initcap(c.region),
		c.state,
		initcap(c.name),
		initcap(s.name),
		d.macaddr,
		d.load_date,
		d.os_distro,
		d.processor,
		simplify_memory(d.memory),
                simplify_hd(d.disk1_size)
	FROM
		(SELECT
			macaddr,
			sch_id,
			max(load_date) AS load_date
		FROM dim_inventory
		WHERE project = proj
		GROUP BY macaddr, sch_id) AS i
	JOIN dim_school s
	ON i.sch_id = s.id
	JOIN dim_city c
	ON 
		s.cit_id = c.id AND
		region = UPPER(region_in) AND
		state = UPPER(state_in) AND
		c.name = UPPER(city_in)
	JOIN dim_inventory d
	ON
		d.sch_id = i.sch_id AND
		d.macaddr = i.macaddr AND
		d.load_date = i.load_date
	ORDER BY s.name, macaddr;
END;
$$ LANGUAGE plpgsql;
